<?php

require_once 'mycasetokens.civix.php';
// phpcs:disable
use CRM_Mycasetokens_ExtensionUtil as E;
// phpcs:enable

/**
 * Implements hook_civicrm_container().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_container/
 */
function mycasetokens_civicrm_container($container) {
  $container->addResource(new \Symfony\Component\Config\Resource\FileResource(__FILE__));
  $container->findDefinition('dispatcher')->addMethodCall('addListener',
    array('civi.token.eval', 'mycasetokens_evaluate_tokens')
  );
}

/**
 * Symfony event for civi.token.eval
 *
 * @param \Civi\Token\Event\TokenValueEvent $event
 */
function mycasetokens_evaluate_tokens(Civi\Token\Event\TokenValueEvent $event) {
  foreach ($event->getRows() as $row) {
    $case_id = null;
    if (isset($row->context['actionSearchResult']) && ($row->context['actionSearchResult']->activity_id)) {
      $case_id = mycasetokens_get_case_id_by_activity_id($row->context['actionSearchResult']->activity_id);
    } elseif (isset($row->context['actionSearchResult']) && ('civicrm_activity' == $row->context['actionSearchResult']->entity_table && $row->context['actionSearchResult']->entity_id)) {
      $case_id = mycasetokens_get_case_id_by_activity_id($row->context['actionSearchResult']->entity_id);
    } elseif (isset($row->context['activityId']) && $row->context['activityId']) {
      $case_id = mycasetokens_get_case_id_by_activity_id($row->context['activityId']);
    }
    if ($case_id) {
      $row->tokens('mycasetokens', 'case_manager', mycasetokens_retrieve_case_manager($case_id));
    }
  }
}


/**
 * Implements hook_civicrm_tokens().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_tokens/
 */
function mycasetokens_civicrm_tokens(&$tokens) {
  $tokens['mycasetokens']['mycasetokens.case_manager'] = E::ts('Case Manager') . ' :: ' . E::ts('My Case Tokens');
}

/**
 * Implements hook_civicrm_buildForm().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_buildForm/
 */
function mycasetokens_civicrm_buildForm($formName, &$form) {
  if ($form instanceof CRM_Activity_Form_Task_Email) {
    \Civi::$statics['mycasetokens']['activity_ids'] = $form->getVar('_activityHolderIds');
  }
}

/**
 * Implements hook_civicrm_tokenValues().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_tokenValues/
 */
function mycasetokens_civicrm_tokenValues(&$values, $contactIDs, $jobID=null, $tokens=array(), $className=null) {
  if (mycasetokens_is_token_present($tokens)) {
    // Normalize $contactIds.
    // $contactIds is either an array with contact ids
    // or it is formatted according $contactIds['contact_id'] = 123, $contactIds['case_id'] = 67
    // if this is the case we wrap the array so it is easier to process.
    if (is_array($contactIDs) && isset($contactIDs['contact_id'])) {
      $contactIDs = [$contactIDs]; // Wrap the $contactIds into an array.
    }
    foreach ($contactIDs as $index => $contact) {
      $contact_id = $contact;
      if (is_array($contact_id) && isset($contact_id['contact_id'])) {
        $contact_id = $contact_id['contact_id'];
      }
      $case_id = mycasetokens_retrieve_case_id($values, $contact, $contact_id);
      if ($case_id) {
        $values[$contact_id]['mycasetokens.case_manager'] = mycasetokens_retrieve_case_manager($case_id);
      }
    }
  }
}

/**
 * Returns true when the {mycasetokens.case_manager} is set and present in the set and
 * needs to be replaced.
 *
 * @param $tokens
 * @return bool
 */
function mycasetokens_is_token_present($tokens) {
  if (in_array('case_manager', $tokens)) {
    return TRUE;
  } elseif (isset($tokens['case_manager'])) {
    return TRUE;
  } elseif (isset($tokens['mycasetokens']) && in_array('case_manager', $tokens['mycasetokens'])) {
    return TRUE;
  } elseif (isset($tokens['mycasetokens']['case_manager'])) {
    return TRUE;
  }
  return FALSE;
}

/**
 * Returns the case manager role name and the display name of the contact who is the manager.
 *
 * @param $case_id
 *
 * @return string
 * @throws \CiviCRM_API3_Exception
 */
function mycasetokens_retrieve_case_manager($case_id) {
  $case = civicrm_api3('Case', 'getsingle', ['id' => $case_id]);
  $caseType = civicrm_api3('CaseType', 'getsingle', ['id' => $case['case_type_id']]);

  $xmlProcessor = new CRM_Case_XMLProcessor_Process();
  $caseRoles = $xmlProcessor->get($caseType['name'], 'CaseRoles');
  [$managerRoleId, $dir] = explode("_", $xmlProcessor->getCaseManagerRoleId($caseType['name']), 2);
  $sql = "
    SELECT `display_name`
    FROM `civicrm_contact`
    INNER JOIN `civicrm_relationship` ON `civicrm_relationship`.`contact_id_b` = `civicrm_contact`.`id`
    WHERE `civicrm_relationship`.`case_id` = %1 AND `civicrm_relationship`.`relationship_type_id` = %2
    AND `is_active` = '1'";
  $sqlParams[1] = [$case_id, 'Integer'];
  $sqlParams[2] = [$managerRoleId, 'Integer'];
  $display_name = CRM_Core_DAO::singleValueQuery($sql, $sqlParams);

  return $caseRoles[$managerRoleId.'_'.$dir].': '.$display_name;
}

/**
 * Return the case_id to be used for replacing the token content.
 *
 * @param $values
 * @param $contacts
 * @param $contact_id
 *
 * @return int|null
 */
function mycasetokens_retrieve_case_id($values, $contacts, $contact_id) {
  if (isset($contacts['case_id'])) {
    return  $contacts['case_id'];
  } elseif (isset($values['activity_id']) && $values['activity_id']) {
    return mycasetokens_get_case_id_by_activity_id($values['activity_id']);
  } elseif (isset($values['extra_data']['activity']['id']) && $values['extra_data']['activity']['id']) {
    return mycasetokens_get_case_id_by_activity_id($values['extra_data']['activity']['id']);
  } elseif (isset(\Civi::$statics['mycasetokens']['activity_ids'])) {
    return mycasetokens_get_case_id_by_contact_id_and_activity_ids($contact_id, \Civi::$statics['mycasetokens']['activity_ids']);
  }
  return null;
}

/**
 * Returns the Case ID retrieved by the activity ID.
 *
 * @param $activity_id
 *
 * @return int|null
 */
function mycasetokens_get_case_id_by_activity_id($activity_id) {
  $case_activity = \Civi\Api4\CaseActivity::get()
    ->addWhere('activity_id', '=', $activity_id)
    ->setLimit(1)->execute()->first();
  return $case_activity['case_id'] ?? NULL;
}

/**
 * Returns the Case ID retrieved by a contact_id and mulitple activity ids.
 * We use this function as we dont know which activity is linked to which contact so
 * we try to retrieve this link again.
 *
 * @param $activity_id
 *
 * @return int|null
 */
function mycasetokens_get_case_id_by_contact_id_and_activity_ids($contact_id, $activity_ids) {
  $case_activity = \Civi\Api4\Activity::get()
    ->addSelect('case.id')
    ->setJoin([
      ['ActivityContact AS activity_contact', 'INNER'],
      ['CaseActivity AS case_activity', 'INNER'],
      ['Case AS case', 'INNER'],
    ])
    ->addWhere('id', 'IN', $activity_ids)
    ->addWhere('activity_contact.contact_id', '=', $contact_id)
    ->addWhere('case.is_deleted', '=', FALSE)
    ->addOrderBy('case.id', 'ASC')
    ->setLimit(1)
    ->execute()
    ->first();
  return $case_activity['case.id'] ?? NULL;
}

/**
 * Implements hook_civicrm_config().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_config/
 */
function mycasetokens_civicrm_config(&$config) {
  _mycasetokens_civix_civicrm_config($config);
}

/**
 * Implements hook_civicrm_xmlMenu().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_xmlMenu
 */
function mycasetokens_civicrm_xmlMenu(&$files) {
  _mycasetokens_civix_civicrm_xmlMenu($files);
}

/**
 * Implements hook_civicrm_install().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_install
 */
function mycasetokens_civicrm_install() {
  _mycasetokens_civix_civicrm_install();
}

/**
 * Implements hook_civicrm_postInstall().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_postInstall
 */
function mycasetokens_civicrm_postInstall() {
  _mycasetokens_civix_civicrm_postInstall();
}

/**
 * Implements hook_civicrm_uninstall().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_uninstall
 */
function mycasetokens_civicrm_uninstall() {
  _mycasetokens_civix_civicrm_uninstall();
}

/**
 * Implements hook_civicrm_enable().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_enable
 */
function mycasetokens_civicrm_enable() {
  _mycasetokens_civix_civicrm_enable();
}

/**
 * Implements hook_civicrm_disable().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_disable
 */
function mycasetokens_civicrm_disable() {
  _mycasetokens_civix_civicrm_disable();
}

/**
 * Implements hook_civicrm_upgrade().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_upgrade
 */
function mycasetokens_civicrm_upgrade($op, CRM_Queue_Queue $queue = NULL) {
  return _mycasetokens_civix_civicrm_upgrade($op, $queue);
}

/**
 * Implements hook_civicrm_managed().
 *
 * Generate a list of entities to create/deactivate/delete when this module
 * is installed, disabled, uninstalled.
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_managed
 */
function mycasetokens_civicrm_managed(&$entities) {
  _mycasetokens_civix_civicrm_managed($entities);
}

/**
 * Implements hook_civicrm_caseTypes().
 *
 * Generate a list of case-types.
 *
 * Note: This hook only runs in CiviCRM 4.4+.
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_caseTypes
 */
function mycasetokens_civicrm_caseTypes(&$caseTypes) {
  _mycasetokens_civix_civicrm_caseTypes($caseTypes);
}

/**
 * Implements hook_civicrm_angularModules().
 *
 * Generate a list of Angular modules.
 *
 * Note: This hook only runs in CiviCRM 4.5+. It may
 * use features only available in v4.6+.
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_angularModules
 */
function mycasetokens_civicrm_angularModules(&$angularModules) {
  _mycasetokens_civix_civicrm_angularModules($angularModules);
}

/**
 * Implements hook_civicrm_alterSettingsFolders().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_alterSettingsFolders
 */
function mycasetokens_civicrm_alterSettingsFolders(&$metaDataFolders = NULL) {
  _mycasetokens_civix_civicrm_alterSettingsFolders($metaDataFolders);
}

/**
 * Implements hook_civicrm_entityTypes().
 *
 * Declare entity types provided by this module.
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_entityTypes
 */
function mycasetokens_civicrm_entityTypes(&$entityTypes) {
  _mycasetokens_civix_civicrm_entityTypes($entityTypes);
}

/**
 * Implements hook_civicrm_thems().
 */
function mycasetokens_civicrm_themes(&$themes) {
  _mycasetokens_civix_civicrm_themes($themes);
}

// --- Functions below this ship commented out. Uncomment as required. ---

/**
 * Implements hook_civicrm_preProcess().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_preProcess
 */
//function mycasetokens_civicrm_preProcess($formName, &$form) {
//
//}

/**
 * Implements hook_civicrm_navigationMenu().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_navigationMenu
 */
//function mycasetokens_civicrm_navigationMenu(&$menu) {
//  _mycasetokens_civix_insert_navigation_menu($menu, 'Mailings', array(
//    'label' => E::ts('New subliminal message'),
//    'name' => 'mailing_subliminal_message',
//    'url' => 'civicrm/mailing/subliminal',
//    'permission' => 'access CiviMail',
//    'operator' => 'OR',
//    'separator' => 0,
//  ));
//  _mycasetokens_civix_navigationMenu($menu);
//}
